@@ -47,6 +47,86 @@
 ;;; 5 ... produce the table structures that are used by ExCCL.
 ;;;
 
+;;;
+;;; MUCS-CCL internal coding system.
+;;;
+
+(defvar mucs-ccl-temporal-coding-system-ccl-symbol nil)
+
+(defsubst mucs-ccl-define-temporal-coding-system (conv)
+  (if (eq mucs-ccl-temporal-coding-system-ccl-symbol
+	  conv)
+      nil
+    (register-ccl-program
+     'mucs-ccl-tmp-ccl-prog
+     (mucs-conversion-get conv 'mucs-compiled-code))
+    (if (not (coding-system-p
+	      'mucs-ccl-temporary))
+	(make-coding-system
+	 'mucs-ccl-temporary
+	 4 ?*
+	 "Temporary coding-system privately used by MUCS-CCL"
+	 '(mucs-ccl-tmp-ccl-prog . mucs-ccl-tmp-ccl-prog)))))
+
+;;; CCL facility checker.
+
+(defvar mucs-ccl-facility-alist
+  '((valid-map-multiple
+     mucs-ccl-check-map-multiple-function
+     "CCL_MapMultiple facility is invalid."
+     none)))
+
+(defun mucs-ccl-check-map-multiple-function ()
+  (let ((vec (make-vector 8 0)))
+    (register-code-conversion-map
+     'mucs-ccl-test-map-1
+     [10 10])
+    (register-code-conversion-map
+     'mucs-ccl-test-map-2
+     [0 1])
+    (ccl-execute
+     (ccl-compile
+      '(0 ((map-multiple
+	    r1 r0 (((mucs-ccl-test-map-1)) mucs-ccl-test-map-2)))))
+     vec)
+    (if (= (aref vec 0) 1)
+	(progn
+;	  (message "!!! This CCL interpreter have the valid MapMultiple instruction.
+;!!! We enable all features on MUCS-CCL.")
+	  t)
+      (message "!!! Sorry. This CCL interpreter have faults on MapMultiple instruction.
+!!! Some features on MUCS-CCL are disabled.")
+      nil)))
+
+(defun mucs-ccl-inspect-facility (required-facility)
+  (let ((slot (assq required-facility mucs-ccl-facility-alist))
+	result)
+    (if (null slot)
+	(error "Unknown facility:%S" required-facility))
+    (setq result (nth 3 slot))
+    (if (eq result 'none)
+	(progn
+	  (setq result (funcall (nth 1 slot)))
+	  (setcar (nthcdr 3 slot) result)))
+    result))
+
+(defun mucs-ccl-get-facility-error-message (facility)
+  (nth 2 (assq facility mucs-ccl-facility-alist)))
+
+(defsubst mucs-ccl-facility-error (required-facility
+				   &optional additional-message)
+  (if (not (mucs-ccl-inspect-facility required-facility))
+      (error (concat
+	      "  !!! "
+	      (mucs-ccl-get-facility-error-message
+	       required-facility)
+	      "\n"
+	      additional-message
+	      "\n"
+	      "If you would like to use this facility, please apply the patch for CCL to
+your Emacs, and MUST recompile ALL PACKAGES of Mule-UCS!
+For details, please refer README file of Mule-UCS"))))
+
 ;;;
 ;;; some simple ccl codes.
 ;;;
@@ -121,7 +201,7 @@
 (defun mucs-ccl-internal-state-reserve (name bit)
   (if (not (symbolp name))
       (signal 'wrong-type-argument
-	      name))
+	      (list 'symbolp name)))
   (if (>= bit (length mucs-ccl-internal-state-usage))
       (error "Bit %d is too large.(%s)" bit name))
   (let ((slot (aref mucs-ccl-internal-state-usage bit)))
@@ -338,15 +418,16 @@ MYO1 and MYO2 is never destructed."
 (defun mucs-ccl-make-elisp-preparation-from-myo (myo)
   (append
    (mucs-ccl-myo-get-elisp myo)
-   (mapcar
-    (function generate-table-set-registration-program)
-    (mucs-ccl-myo-get-table-set myo))))
+   (if mucs-current-package
+       (mapcar
+	(function generate-table-set-registration-program)
+	(mucs-ccl-myo-get-table-set myo)))))
 
 ;;;
 ;;; MUCS Conversion Engine API.
 ;;;
 
-(defsubst mucs-convert (conversion object)
+(defun mucs-convert (conversion object)
   (let ((ccl-object
 	 (funcall
 	  (mucs-type-get-ccl-representation
@@ -361,7 +442,8 @@ MYO1 and MYO2 is never destructed."
 		       ((vectorp ccl-object)
 			ccl-object)
 		       (t
-			(signal 'mucs-ccl-invalid-error object))))
+			(signal 'mucs-ccl-invalid-error
+				(list object)))))
     (if (= (aref execute-vector 0) mucs-invalid-code)
 	nil
       (funcall
@@ -371,6 +453,66 @@ MYO1 and MYO2 is never destructed."
 	   execute-vector
 	 (aref execute-vector 0))))))
 
+(defun mucs-convert-region (start end conversion)
+  "Convert the current region by specified conversion,
+which is defined by `mucs-define-conversion'.
+It returns the length of the encoded text."
+  (interactive 
+   (list (region-beginning)
+	 (region-end)
+	 (intern
+	  (completing-read
+	   "MUCS Conversion:"
+	   (let (candlist)
+	     (mapatoms
+	      (lambda (x)
+		(if (and (mucs-conversion-p x)
+			 (eq (mucs-conversion-get-conv-type x)
+			     'stream))
+		    (setq candlist
+			  (cons
+			   (cons (symbol-name x) 0)
+			   candlist)))))
+	     candlist)))))
+  (if (not (and (mucs-conversion-p conversion)
+		(eq (mucs-conversion-get-conv-type
+		     conversion)
+		    'stream)))
+      (error "%S is not MUCS conversion" conversion))
+  (mucs-ccl-define-temporal-coding-system conversion)
+  (encode-coding-region start end 'mucs-ccl-temporary))
+
+(defun mucs-convert-string (string conversion &optional unibyte-p)
+  "Convert STRING by specified conversion,
+which is defined by `mucs-define-conversion'.
+It returns the encoded text.
+
+If UNIBYTE-P is non-nil, return a unibyte string as a result."
+  (if (not (stringp string))
+      (signal 'wrong-type-argument
+	      (list 'stringp string)))
+  (if (not (and (mucs-conversion-p conversion)
+		(eq (mucs-conversion-get-conv-type
+		     conversion)
+		    'stream)))
+      (error "%S is not MUCS conversion" conversion))
+  (ccl-execute-on-string conversion (make-vector 9 nil)
+			 string nil unibyte-p))
+
+(defun mucs-ccl-setup-myo (config)
+  (let ((myo (mucs-ccl-empty-myo))
+	ret)
+    (while config
+      (setq ret (eval (car config))
+	    config (cdr config))
+      (if (consp ret)
+	  (if (mucs-ccl-myo-p ret)
+	      (setq myo (mucs-ccl-myo-append myo ret))
+	    (setq myo (mucs-ccl-myo-add-ccl ret myo)))
+	(if ret
+	    (error "Invalid MUCS-CCL:%S" ret))))
+    myo))
+
 (defun mucs-setup-conversion (conv def)
   (let ((convtype (mucs-conversion-get-conv-type conv))
 	(mucs-current-conversion conv)
@@ -378,32 +520,20 @@ MYO1 and MYO2 is never destructed."
 	(mag (mucs-conversion-definition-mag def))
 	(config-main (mucs-conversion-definition-main-prog def))
 	(config-eof (mucs-conversion-definition-eof-prog def))
-	(main-ccl-myo (mucs-ccl-empty-myo))
-	(eof-ccl-myo (mucs-ccl-empty-myo))
+	main-ccl-myo eof-ccl-myo
 	ccl-program
-	compiled-ccl
-	func ret)
-    (setq func (lambda (config myo)
-		 (while config
-		   (setq ret (eval (car config))
-			 config (cdr config))
-		   (if (consp ret)
-		       (if (mucs-ccl-myo-p ret)
-			   (setq myo (mucs-ccl-myo-append myo ret))
-			 (setq myo (mucs-ccl-myo-add-ccl ret myo)))
-		     (if ret
-			 (error "Invalid MUCS-CCL:%S" ret))))))
+	compiled-ccl)
 
     (if (consp convtype)
 	(setq mucs-current-type (car convtype)))
-    (funcall func config-main main-ccl-myo)
+    (setq main-ccl-myo (mucs-ccl-setup-myo config-main))
     (if (and (consp convtype)
 	     (not (eq (cdr convtype) mucs-current-type)))
 	(signal 'mucs-type-mismatch-error
 		(cons mucs-current-type (cdr convtype))))
     (if (consp convtype)
 	(setq mucs-current-type (car convtype)))
-    (funcall func config-eof eof-ccl-myo)
+    (setq eof-ccl-myo (mucs-ccl-setup-myo config-eof))
 ;;; EOF CCL program is not executed when non-stream conversion.
 ;    (if (and (consp convtype)
 ;	     (not (eq (cdr convtype) mucs-current-type)))
@@ -486,6 +616,44 @@ MYO1 and MYO2 is never destructed."
 (defun mucs-ccl-if-invalid-repeat ()
   (mucs-ccl-if-invalid '(quote (((repeat))))))
 
+;;
+;; CCL registration APIs.
+;;
+
+(defun mucs-ccl-register-ccl-program (name ccl-block)
+  (mucs-register-object 'mucs-ccl-required name)
+;  (message "MCCLREG(%S):%S" name ccl-block)
+  (put name
+       'mucs-ccl-required-ccl-program
+       ccl-block))
+
+(defun mucs-ccl-generate-registration-program ()
+  (let ((ccl-pgm-list
+	 (mucs-unembedded-object-list 'mucs-ccl-required))
+	name myo result)
+    (while (setq name (car ccl-pgm-list))
+      (setq myo (mucs-ccl-setup-myo
+		 (get name 'mucs-ccl-required-ccl-program)))
+      (setq result
+	    (append
+	     result
+	     (mucs-ccl-make-elisp-preparation-from-myo myo)
+	     `((register-ccl-program
+		',name
+		,(ccl-compile 
+		  (list 0
+			(mucs-ccl-myo-get-ccl myo)))))))
+      (setq ccl-pgm-list (cdr ccl-pgm-list)))
+;   (message "MCCLREGFIN:%S" result)
+    `(progn
+       (setq mucs-ccl-facility-alist
+	     (quote ,mucs-ccl-facility-alist))
+       ,@result)))
+
+;;; Add hook for embedding translation informations to a package.
+(add-hook 'mucs-package-definition-end-hook
+	  (function mucs-ccl-generate-registration-program))
+
 ;;
 ;; Common I/O interface.
 ;;
@@ -502,4 +670,27 @@ MYO1 and MYO2 is never destructed."
 	mucs-current-type serialize nil))
     (error "Current TYPE is not set.  Cannot write.")))
 
+;;
+;; Short CCL routine.
+;;
+
+(defun mucs-ccl-add-program (reg add)
+  (if (not (symbolp reg))
+      (signal 'wrong-type-argument
+	      (list 'symbolp reg)))
+  (if (not (numberp add))
+      (signal 'wrong-type-argument
+	      (list 'numberp add)))
+  (let ((sym
+	 (intern (format "mucs-ccl-add-%S-%d" reg add))))
+    (if (not (mucs-registered-p 'mucs-ccl-required sym))
+	(progn
+	  (mucs-ccl-register-ccl-program
+	   sym
+	   `((quote ((,reg += ,add)))))
+	  (register-ccl-program
+	   sym
+	   (ccl-compile `(0 ((,reg += ,add)))))))
+    sym))
+
 (provide 'mucs-ccl)
