@@ -1,19 +1,19 @@
 ;;; -*- coding: iso-2022-7bit  -*-
-;;; mucs-type.el --- MULE-UCS type and representaion management library
+;;; mucs-type.el --- Mule-UCS type and representaion management library
 
 ;; Copyright (C) 1999 Miyashita Hisashi
 
 ;; Keywords: mule, multilingual, 
 ;;           character set, coding-system, ISO10646, Unicode
 
-;; This file is part of MULE-UCS
+;; This file is part of Mule-UCS
 
-;; MULE-UCS is free software; you can redistribute it and/or modify
+;; Mule-UCS is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; MULE-UCS is distributed in the hope that it will be useful,
+;; Mule-UCS is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
@@ -33,15 +33,15 @@
 ;;;
 ;;;    But, like XEmacs, some systems distinguish between number and
 ;;;    character.  In addition, Emacs may give some abstraction
-;;;    to characters.  Therefore MULE-UCS should prepare certain
+;;;    to characters.  Therefore Mule-UCS should prepare certain
 ;;;    abstraction layer to represent character format on CCL.
 
-;;;    MULE-UCS prepare `TYPE' to give any kind of representations
+;;;    Mule-UCS prepare `TYPE' to give any kind of representations
 ;;;    defined by users to data sturucture to be translated.
 ;;;    Users can define any representation on translation in their fashion.
 
 ;;; TYPE specification.
-;;;      TYPE on MULE-UCS is for specifying character data categories that
+;;;      TYPE on Mule-UCS is for specifying character data categories that
 ;;; are distinguished one another whenever we treat it in CCL environment
 ;;; EmacsLisp character representation, and some other environments(where
 ;;; data are stored in serialized format).
@@ -81,13 +81,15 @@
 
 (defsubst mucs-type-get-option-alist (type)
   (nth 4 (or (assq type mucs-type-alist)
-	     (signal 'mucs-unknown-type-error type))))
+	     (signal 'mucs-unknown-type-error
+		     (list type)))))
 
 (defsubst mucs-type-set-option-alist (type alist)
   (let ((slot (assq type mucs-type-alist)))
     (if slot
 	(setcar (nthcdr 4 slot) alist)
-      (signal 'mucs-unknown-type-error type))))
+      (signal 'mucs-unknown-type-error
+	      (list type)))))
 
 (defsubst mucs-type-assq-option-alist (type key)
   (assq key (mucs-type-get-option-alist type)))
@@ -151,28 +153,32 @@
   (let ((slot (assq type
 		    mucs-type-alist)))
     (if (null slot)
-	(signal 'mucs-unknown-type-error type)
+	(signal 'mucs-unknown-type-error
+		(list type))
       (car (nth 1 slot)))))
 
 (defun mucs-type-get-ccl-representation (type)
   (let ((slot (assq type
 		    mucs-type-alist)))
     (if (null slot)
-	(signal 'mucs-unknown-type-error type)
+	(signal 'mucs-unknown-type-error
+		(list type))
       (cdr (nth 1 slot)))))
 
 (defun mucs-type-get-equal-op (type)
   (let ((slot (assq type
 		    mucs-type-alist)))
     (if (null slot)
-	(signal 'mucs-unknown-type-error type)
+	(signal 'mucs-unknown-type-error
+		(list type))
       (car (nth 2 slot)))))
 
 (defun mucs-type-get-relative-op (type)
   (let ((slot (assq type
 		    mucs-type-alist)))
     (if (null slot)
-	(signal 'mucs-unknown-type-error type)
+	(signal 'mucs-unknown-type-error
+		(list type))
       (cdr (nth 2 slot)))))
 
 
@@ -191,7 +197,8 @@
 ;		     (eval serialize)
 ;		     t)
 ;		 (error nil))))
-      (signal 'mucs-invalid-serialization-error serialize)))
+      (signal 'mucs-invalid-serialization-error
+	      (list serialize))))
 
 (defun mucs-type-register-serialization (type name serialize unserialize)
   (if (not (symbolp name))
@@ -205,7 +212,8 @@
 	serial-slot
 	serials)
     (if (null slot)
-	(signal 'mucs-unknown-type-error type))
+	(signal 'mucs-unknown-type-error
+		(list type)))
     (setq serials (nth 3 slot)
 	  serial-slot (assq name serials)
 	  cell (cons name (cons serialize unserialize)))
@@ -246,7 +254,8 @@
 	to-type-conv-slot
 	conv-func-slot)
     (if (null slot)
-	(signal 'mucs-unknown-type-error from-type))
+	(signal 'mucs-unknown-type-error
+		(list from-type)))
     (mucs-representation-check-conversion-function conversion)
 
     (setq to-type-conv-slot (mucs-type-assq-option-alist
@@ -270,7 +279,8 @@
 	to-type-conv-slot
 	conv-func-slot)
     (if (null slot)
-	(signal 'mucs-unknown-type-error from-type))
+	(signal 'mucs-unknown-type-error
+		(list from-type)))
 
     (setq to-type-conv-slot (mucs-type-assq-option-alist
 			     from-type 'type-conversion)
@@ -296,7 +306,8 @@ If convertion cannot be found, cause error."
 (defun mucs-type-serialization-list (type)
   (let ((slot (assq type mucs-type-alist)))
     (if (null slot)
-	(signal 'mucs-unknown-type-error type))
+	(signal 'mucs-unknown-type-error
+		(list type)))
     (mapcar
      'car
      (nth 3 slot))))
