@@ -6,14 +6,14 @@
 ;; Keywords: mule, multilingual, 
 ;;           character set, coding-system, ISO10646, Unicode
 
-;; This file is part of MULE-UCS
+;; This file is part of Mule-UCS
 
-;; MULE-UCS is free software; you can redistribute it and/or modify
+;; Mule-UCS is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; MULE-UCS is distributed in the hope that it will be useful,
+;; Mule-UCS is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
@@ -26,7 +26,6 @@
 ;; Comment:
 
 ;;;; 
-;; 0.70->KITANOTENMANGU Shrine
 ;; 0.80->JISHOJI Temple
 ;; 0.90->Mt.DAIMONJI
 ;; 1.00->SHINNYODO Temple
@@ -40,7 +39,7 @@
 ;; 1.80->IMADEGAWA
 ;; 1.90->DAIRI
 ;; 2.00->Kyoto Univ.
-(defconst mucs-version "0.74 (KAMIGAMO:$B>e2lLP(B)")
+(defconst mucs-version "0.80 (KITAYAMA:$BKL;3(B)")
 
 ;; For error handling.
 
@@ -89,7 +88,7 @@
   "invalid code.  If this value is set, skip operation.")
 
 (defun mucs-max-code ()
-  (1- (lsh 1 (1- mucs-code-range-bits))))
+  (1- (lsh 1 mucs-code-range-bits)))
 
 (defun mucs-special-code (code)
   (if (or (< code 0)
@@ -100,6 +99,16 @@
 (defun mucs-arithmetic-adjust ()
   (* 3 (lsh 1 (- mucs-code-range-bits 2))))
 
+(defun mucs-arithmetic-range-lower ()
+  (lsh 1 (1- mucs-code-range-bits)))
+
+(defun mucs-arithmetic-range-upper ()
+  (- (lsh 1 mucs-code-range-bits)
+     mucs-code-range-specials 1))
+
+(defun mucs-max-number ()
+  (1- (lsh 1 (1- mucs-code-range-bits))))
+
 (defun mucs-number-mask ()
   (lognot 0))
 
@@ -179,12 +188,13 @@ In order to embed data or lisp code, use this hook.")
 
 (defmacro mucs-define-package (package &rest form)
   "Enclose a unit of package with this.
-By this specification, MULE-UCS may judge
+By this specification, Mule-UCS may judge
 whether generate a new program to prepare.
 You should make PACKAGE the same as your package name
 that you set at `provide' function."
   (if (not (symbolp package))
-      (signal 'wrong-type-argument package))
+      (signal 'wrong-type-argument
+	      (list 'symbolp package)))
   (setq mucs-current-package package)
   (put mucs-current-package 'mucs-registered-alist nil)
   (append
@@ -253,7 +263,7 @@ that you set at `provide' function."
 	 (mucs-get-registered-kind-alist kind))
 	elem result)
     (while (setq elem (car objlist))
-      (if (not (cdr elem))
+      (if (not (nth 1 elem))
 	  (setq result (cons (car elem) result)))
       (setq objlist (cdr objlist)))
     result))
@@ -314,29 +324,49 @@ otherwise return nil."
 ;;
 ;; PROPERTY SYMBOL LIST
 ;;    mucs-conv-type:
-;;    mucs-conversion-definition:
-;;    mucs-dependent-translation:
-;;    mucs-compiled-code(optional):
+;;    mucs-conversion-program:
+;;    mucs-conversion-properties:
+;;    mucs-conversion-program-marker:
 
 (defvar mucs-current-conversion nil)
 
+(defsubst mucs-conversion-p (symbol)
+  (or (get symbol 'mucs-conv-type)
+      nil))
+
+(defsubst mucs-conversion-get (symbol key)
+  (if (not (mucs-conversion-p symbol))
+      (error "%S is not mucs-conversion." symbol))
+  (plist-get (get symbol 'mucs-conversion-properties)
+	     key))
+
+(defsubst mucs-conversion-put (symbol key val)
+  (if (not (mucs-conversion-p symbol))
+      (error "%S is not mucs-conversion." symbol))
+  (put symbol
+       'mucs-conversion-properties
+       (plist-put (get symbol 'mucs-conversion-properties)
+		  key val)))
+
 (defmacro mucs-define-conversion (symbol convtype definition)
   "Define conversion.
 SYMBOL is a symbol to identify the defined conversion.
 CONVTYPE specifies how this conversion is used; You can specify
-either stream(symbol) or (FROM-TYPE . TO-TYPE), and FROM-TYPE and TO-TYPE
-are defined MULE-UCS-TYPE.
+stream(symbol), font(symbol), or (FROM-TYPE . TO-TYPE),
+where FROM-TYPE and TO-TYPE are defined MULE-UCS-TYPE.
   If CONVTYPE is stream, this conversion is used for stream, i.e.
 this can be used by coding-system.
+  If CONVTYPE is font, this conversion is used for font encoding.
   If CONVTYPE is (FROM-TYPE . TO-TYPE), this conversion is used for
 converting data of FROM-TYPE into data of TO-TYPE.
   DEFINITION specifies the definition of the conversion.
 
   conversions defined by this function are embedded to .elc file.
-Therefore, you can use these without any MULE-UCS modules.
+Therefore, you can use these without any Mule-UCS modules.
 
   All arguments are NOT evaluated!"
   (if (not (or (eq convtype 'stream)
+	       (eq convtype 'font)
 	       (consp convtype)
 	       (mucs-type-p (car convtype))
 	       (mucs-type-p (cdr convtype))))
@@ -346,27 +376,30 @@ Therefore, you can use these without any MULE-UCS modules.
      (put (quote ,symbol) 'mucs-conv-type (quote ,convtype))
      ,@(mucs-setup-conversion symbol definition)
      (put (quote ,symbol) 'mucs-conversion-program
-	  ,(get symbol 'mucs-conversion-program-prep))
+	  ,(mucs-conversion-get
+	    symbol 'mucs-conversion-program-prep))
      nil))
 
 (defun mucs-conversion-set-program-marker (marker-sym program)
   (list '\, `(cdar (put (quote ,mucs-current-conversion)
 			'mucs-conversion-program-marker
 			(cons (cons (quote ,marker-sym)
-				    (quote ,program))
+				    ,(list '\` program))
 			      (get (quote ,mucs-current-conversion)
 				   'mucs-conversion-program-marker))))))
 
 (defsubst mucs-retrieve-marked-conversion-program (conv mark)
   (cdr (assq mark (get conv 'mucs-conversion-program-marker))))
 
-(defun mucs-modify-conversion (conv mark newprog)
+(defsubst mucs-substitute-conversion-program (conv mark newprog)
   (let ((spot (mucs-retrieve-marked-conversion-program conv mark)))
     (setcar spot (car newprog))
-    (setcdr spot (cdr newprog))
-    (mucs-refresh-conversion
-     conv
-     (get conv 'mucs-conversion-program))))
+    (setcdr spot (cdr newprog))))
+
+(defun mucs-modify-conversion (conv mark newprog)
+  (mucs-substitute-conversion-program conv mark newprog)
+  (mucs-refresh-conversion
+   conv (get conv 'mucs-conversion-program)))
   
 (defun mucs-conversion-definition-mag (definition)
   (eval (car definition)))
@@ -382,9 +415,11 @@ Therefore, you can use these without any MULE-UCS modules.
 
 (defsubst mucs-conversion-set-program-and-compiled-code
   (symbol program code)
-  (put symbol 'mucs-conversion-program-prep program)
+  (mucs-conversion-put symbol 'mucs-conversion-program-prep program)
   (if code
-      (put symbol 'mucs-compiled-code code)))
+      (mucs-conversion-put
+       symbol
+       'mucs-compiled-code code)))
 
 ;;;
 ;;;
