@@ -23,7 +23,7 @@
 ;; Boston, MA 02111-1307, USA.
 
 ;;; This module manages Translation And Encodings.
-;;; This is main and very important module to MULE-UCS.
+;;; This is main and very important module to Mule-UCS.
 ;;; TAE is CORE module. But this is VERY insufficient version.
 
 (require 'mucs)
@@ -43,13 +43,17 @@
 	 (intern (format "tae-project-all-to-%d" val)))
 	((eq val 'invalid)
 	 'tae-project-all-to-invalid)
+	((eq val t)
+	 'tae-project-all-to-restore)
 	(t
-	 (signal 'wrong-type-argument val))))
+	 (signal 'wrong-type-argument
+		 (list val)))))
 
 (defun tae-generate-special-project-table-symbol (code)
   (if (numberp code)
       (intern (format "tae-project-special-%d-to-identity" code))
-    (signal 'wrong-type-argument code)))
+    (signal 'wrong-type-argument
+	    (list code))))
 
 (defun tae-project-all-table (val)
   (let ((sym (tae-generate-project-table-symbol val)))
@@ -60,38 +64,48 @@
        (vector t
 	       (if (eq val 'invalid)
 		   mucs-invalid-code val)
-	       0 (mucs-max-code)))
+	       0 (mucs-max-number)))
       sym)))
 
 (defun tae-project-all-arith-to-lambda ()
-  (if (get 'tae-project-all-arith-to-lambda
-	   'code-conversion-map)
-      'tae-project-all-arith-to-lambda
-    (register-code-conversion-map
-     'tae-project-all-special-to-identity
-     (vector t 'lambda
-	     (mucs-special-code 0)
-	     (1+ (mucs-special-code
-		  (1- mucs-code-range-specials)))))))
+  (if (null (get 'tae-project-all-arith-to-lambda
+		 'code-conversion-map))
+      (register-code-conversion-map
+       'tae-project-all-arith-to-lambda
+       (vector t 'lambda
+	       (mucs-arithmetic-range-lower)
+	       (1+ (mucs-arithmetic-range-upper)))))
+  'tae-project-all-arith-to-lambda)
+
+(defun tae-force-project-to-lambda ()
+  (if (null (get 'tae-force-project-to-lambda
+		 'code-conversion-map))
+      (register-code-conversion-map
+       'tae-force-project-to-lambda
+       (vector t 'lambda
+	       (mucs-special-code 0)
+	       (mucs-max-code))))
+  'tae-force-project-to-lambda)
 
 (defun tae-project-special-to-identity-table (code)
   (let ((sym (tae-generate-project-table-symbol code)))
-    (if (get sym 'code-conversion-map)
-	sym
-      (register-code-conversion-map
-       sym
-       (vector (mucs-special-code code) t)))))
+    (if (null (get sym 'code-conversion-map))
+	(register-code-conversion-map
+	 sym
+	 (vector (mucs-special-code code) t)))
+    sym))
+      
 
 (defun tae-project-all-specials-to-identity-table ()
-  (if (get 'tae-project-all-special-to-identity
-	   'code-conversion-map)
-      'tae-project-all-special-to-identity
-    (register-code-conversion-map
-     'tae-project-all-special-to-identity
-     (vector t t
-	     (mucs-special-code 0)
-	     (1+ (mucs-special-code
-		  (1- mucs-code-range-specials)))))))
+  (if (null (get 'tae-project-all-special-to-identity
+		 'code-conversion-map))
+      (register-code-conversion-map
+       'tae-project-all-special-to-identity
+       (vector t t
+	       (mucs-special-code 0)
+	       (1+ (mucs-special-code
+		    (1- mucs-code-range-specials))))))
+  'tae-project-all-special-to-identity)
 
 (defun tae-project-all-to-special-table (code)
   (tae-project-all-table (mucs-special-code code)))
@@ -380,7 +394,7 @@ after its compilation."
 ;;;
 ;;; TYPE := registered type (see mucs-types.el)
 ;;; OBJECT := Lisp Object
-;;;  (that mucs be valid in terms of the specified TYPE) (see mucs-types.el)
+;;;  (that must be valid in terms of the specified TYPE) (see mucs-types.el)
 ;;;
 ;;;
 ;;; TRANSLATION := TRANSLATION-ELEMENT  |
@@ -669,7 +683,7 @@ When DECODEP is t, reduce for decodeing translation."
   (let ((tr1 (car tr-elems))
 	(tr2 (nth 1 tr-elems))
 	(tr-curl (cdr tr-elems))
-	result-list result)
+	result-list result tmp)
     (while (and tr2
 		(not (tae-get-tr-elem-all-key tr1)))
       (setq result
@@ -736,6 +750,22 @@ When DECODEP is t, reduce for decodeing translation."
 
 		     result))
 
+		  ((or (and (tae-tr-elem-assoc-p tr1)
+			    (tae-tr-elem-range-p tr2))
+		       (and (tae-tr-elem-assoc-p tr2)
+			    (tae-tr-elem-range-p tr1)
+			    (setq tmp tr2
+				  tr2 tr1
+				  tr1 tmp)))
+		   ;; tr1 ... assoc tr element
+		   ;; tr2 ... range tr element
+		   nil)
+
+		  ((and (tae-tr-elem-range-p tr1)
+			(tae-tr-elem-range-p tr2))
+		   ;; tr1, tr2 ... range tr elements
+		   nil)
+
 		  ((or (or (tae-tr-elem-ccl-p tr1)
 			   (tae-tr-elem-range-p tr1))
 		       (or (tae-tr-elem-ccl-p tr2)
@@ -1016,16 +1046,16 @@ according to it, rearrange the elements of UNIT."
 		  (if (= (length args) 1)
 		      (car args)
 		    (cons 'c args)))
-		 ((eq op 'ct)
-		  (setq args (tae-normalize-reduction-unit args decodep))
-		  (setq args 
-			(tae-normalize-reduction-unit
-			 (tae-reduce-composite-transparent-translations
-			  args decodep)
-			 decodep))
-		  (if (= (length args) 1)
-		      (car args)
-		    (cons 'ct args)))
+; 		 ((eq op 'ct)
+; 		  (setq args (tae-normalize-reduction-unit args decodep))
+; 		  (setq args 
+; 			(tae-normalize-reduction-unit
+; 			 (tae-reduce-composite-transparent-translations
+; 			  args decodep)
+; 			 decodep))
+; 		  (if (= (length args) 1)
+; 		      (car args)
+; 		    (cons 'ct args)))
 		 (t
 		  (error "TAE have not supported OP:`%S' yet!!" op)))))))
 
@@ -1103,13 +1133,21 @@ according to it, rearrange the elements of UNIT."
 		     message)))
     message))
 
-(defsubst tae-message-set-new-type (message type)
+(defsubst tae-message-set-new-type (message type &optional orig-type)
   (let ((slot (assq 'new-type message)))
     (if slot
 	(progn
-	  (setcdr slot type)
+	  (cond (type
+		 (setcdr slot (list type orig-type)))
+		(orig-type
+		 (setcdr (cdr slot) (list orig-type))))
 	  message)
-      (cons (cons 'new-type type) message))))
+      (cons (list 'new-type type orig-type) message))))
+
+(defsubst tae-message-get-new-type (message &optional origp)
+  (let ((slot (assq 'new-type message)))
+    (and slot
+	 (if origp (nth 2 slot) (nth 1 slot)))))
 
 (defsubst tae-message-set-arithmetic (message flag)
   (let ((slot (assq 'arithmetic message)))
@@ -1243,18 +1281,26 @@ according to it, rearrange the elements of UNIT."
 			    (if (/= (- end1 st1) (- end2 st2))
 				(error "Inconsistent range slot:%S(%d, %d)-(%d, %d)"
 				       elem st1 end1 st2 end2))))
-		     (setq tables
-			    (cons
-			     (vector t (+ (- st2 st1)
-					  (mucs-arithmetic-adjust))
-				     st1 (1+ end1))
-			     tables)))
+		     (if (mucs-ccl-inspect-facility 'valid-map-multiple)
+			 (setq tables
+			       (cons
+				(vector t (mucs-ccl-add-program
+					   'r0 (- st2 st1))
+					st1 (1+ end1))
+				tables))
+		       (setq tables
+			     (cons
+			      (vector t (+ (- st2 st1)
+					   (mucs-arithmetic-adjust))
+				      st1 (1+ end1))
+			      tables))))
 		   (list 
 		    (cons 'tables
 			  (nreverse tables))
 		    '(arithmetic . t))))))
 
-    (tae-message-set-new-type result to-type)))
+    (tae-message-set-new-type result
+			      to-type from-type)))
 
 (defun tae-compile-OR-operation (translations type decodep)
   (let (tr mes-tbl mes tbls result-mes to-type new-type arith)
@@ -1264,13 +1310,14 @@ according to it, rearrange the elements of UNIT."
 		     tr type decodep)
 	    mes (car mes-tbl)
 	    tbls (nconc tbls (cdr mes-tbl))
-	    new-type (cdr (assq 'new-type mes))
+	    new-type (tae-message-get-new-type mes)
 	    arith (or arith (tae-message-arithmetic-p mes)))
 
       ;;; operate type.
       (if to-type
 	  (if (not (eq to-type new-type))
-	      (signal 'mucs-type-mismatch-error to-type new-type))
+	      (signal 'mucs-type-mismatch-error
+		      (list to-type new-type)))
 	(setq to-type new-type)
 	(setq result-mes
 	      (tae-message-set-new-type result-mes to-type)))
@@ -1283,7 +1330,9 @@ according to it, rearrange the elements of UNIT."
      arith)))
 
 (defun tae-compile-AND-operation (translations type decodep)
-  (error "Sorry!. Compilation of AND operation is disabled intentionally.")
+  (mucs-ccl-facility-error
+   'valid-map-multiple
+   "Sorry.  AND operation on compilation is disabled.")
   (let (tr mes-tbl mes tbls result-mes to-type new-type
 	   arith first-tbl)
     (while (setq tr (car translations))
@@ -1291,39 +1340,40 @@ according to it, rearrange the elements of UNIT."
 	    mes-tbl (tae-get-message-from-translation
 		     tr type decodep)
 	    mes (car mes-tbl)
-	    new-type (cdr (assq 'new-type mes)))
+	    new-type (tae-message-get-new-type mes))
 
       ;;; operate type.
       (if to-type
 	  (if (not (eq to-type new-type))
-	      (signal 'mucs-type-mismatch-error to-type new-type))
+	      (signal 'mucs-type-mismatch-error
+		      (list to-type new-type)))
 	(setq to-type new-type)
 	(setq result-mes
 	      (tae-message-set-new-type result-mes to-type)))
       (if first-tbl
-	  (setq tbls (nconc (list
-			     (list (nconc (cdr mes-tbl)
-					  (list (tae-project-all-to-special-table 1)))
-				   (tae-project-all-table t))
-			     (tae-project-special-to-identity-table 1))
-			    tbls))
-	(setq first-tbl (cdr mes-tbl)
+	  (setq tbls (cons (list (cdr mes-tbl)
+				 (tae-project-all-table t))
+			   tbls))
+	(setq first-tbl (list (cdr mes-tbl))
+	      tbls first-tbl
 	      arith (tae-message-arithmetic-p mes)))
 
       ;; normal message keys are preserved.
       (setq result-mes (tae-message-append result-mes mes)))
-    (setq tbls (list tbls first-tbl))
     (tae-message-set-arithmetic
      (tae-message-set-tables result-mes
 			     tbls)
      arith)))
 
 (defun tae-compile-composite-operation (translations type decodep)
-  (error "Sorry!.  In this version, compilation of composite operation
-is disabled intentionally.")
-  (if decodep (setq translations (reverse translations)))
-  (let (tr mes-tbl mes tbls result-mes to-type new-type
-	   arith)
+  (mucs-ccl-facility-error
+   'valid-map-multiple
+   "Sorry.  Composite operation on compilation is disabled.")
+  (let (mes-tbl mes
+	tbls result-mes
+	new-type
+	tr arith)
+    (if decodep (setq translations (reverse translations)))
     (setq tr (car translations)
 	  translations (cdr translations))
     (if tr
@@ -1332,29 +1382,33 @@ is disabled intentionally.")
 	      (setq mes-tbl (tae-get-message-from-translation
 			     tr type decodep)
 		    mes (car mes-tbl)
-		    new-type (cdr (assq 'new-type mes))
+		    new-type (tae-message-get-new-type mes)
 		    arith (tae-message-arithmetic-p mes))
 
 	      ;; operate type.
-	      (if to-type
-		  (if (not (eq to-type new-type))
-		      (signal 'mucs-type-mismatch-error to-type new-type))
-		(setq to-type new-type)
-		(setq result-mes
-		      (tae-message-set-new-type result-mes to-type)))
+	      (setq type new-type)
+	      (setq result-mes
+		    (tae-message-set-new-type result-mes type))
+
 	      ;; normal message keys are preserved.
 	      (setq result-mes (tae-message-append result-mes mes))
 	      translations)
+
+	  ;; build up table structure.
 	  (setq tbls (nconc tbls
-			    (list (nconc (cdr mes-tbl)
-					 (list (tae-project-all-to-special-table 1))))
-			    (if arith (list (tae-project-all-arith-to-lambda))))
-		tr (car translations)
+			    (list (cdr mes-tbl))
+;			    (if arith (list (tae-project-all-arith-to-lambda)
+			    ))
+
+	  (setq	tr (car translations)
 		translations (cdr translations))))
 
-    (setq tbls (nconc tbls
-		      (list (cdr mes-tbl)
-			    (list (tae-project-special-to-identity-table 1)))))
+    (setq tbls (list
+		(nconc tbls
+		       (list (cdr mes-tbl))
+;		       (if arith (list (tae-project-all-arith-to-lambda)))
+		       )
+		(tae-project-all-table t)))
     (tae-message-set-arithmetic
      (tae-message-set-tables result-mes
 			     tbls)
@@ -1370,7 +1424,7 @@ composite-transparent operation is disabled intentionally.")
 		    (car translations) type decodep))
 	mes-tbl-b mes tbls result-mes)
     (setq mes (car mes-tbl-a)
-	  type (cdr (assq 'new-type mes))
+	  type (tae-message-get-new-type mes)
 	  mes-tbl-b (tae-get-message-from-translation
 		     (nth 1 translations) type decodep)
 	  tbls (cons (cdr mes-tbl-a)
@@ -1414,8 +1468,8 @@ composite-transparent operation is disabled intentionally.")
 	   (tae-compile-AND-operation args type decodep))
 	  ((eq op 'c)
 	   (tae-compile-composite-operation args type decodep))
-	  ((eq op 'ct)
-	   (tae-compile-composite-transparent-operation args type decodep))
+;	  ((eq op 'ct)
+;	   (tae-compile-composite-transparent-operation args type decodep))
 	  (t
 	   (error "TAE have not supported OP:`%S' yet!!" op)))))
 
@@ -1458,8 +1512,8 @@ composite-transparent operation is disabled intentionally.")
 ;;;
 ;;; Persistent data for translation rule.
 ;;;   `Persistent' means data that is necessary
-;;; to manipulate the tranlation rule is kept in
-;;; byte-compiled file.
+;;; to manipulate the tranlation rule for storing into
+;;; a byte-compiled file.
 
 (defun tae-persistent-program-template (name item objs replacep)
   ;;(message "NAME:%S ITEM:%S OBJS:%S" name item objs)
@@ -1601,10 +1655,11 @@ composite-transparent operation is disabled intentionally.")
 	(tae-register-conversion
 	 name type decodep
 	 mucs-current-conversion))
+    ;; We should not directly access message-slots...
     (while (setq mes-el (car mes-cur))
       (setq mes-cur (cdr mes-cur))
       (cond ((eq 'new-type (car mes-el))
-	     (setq mucs-current-type (cdr mes-el)))
+	     (setq mucs-current-type (nth 1 mes-el)))
 
 	    ((eq 'arithmetic (car mes-el))
 	     ;; do nothing.
@@ -1627,22 +1682,36 @@ composite-transparent operation is disabled intentionally.")
      table-syms-nested-p)))
 
 (defun tae-compile-retrieve-ccl-prog (table-syms nestedp)
-  (if nestedp
+  (if (mucs-ccl-inspect-facility 'valid-map-multiple)
+      (if nestedp
+	  `(loop
+	    (r1 = 0)
+	    (r4 = r0)
+	    (map-multiple r1 r0 ,table-syms))
+	`(loop
+	  (r1 = 0)
+	  (r4 = r0)
+	  (iterate-multiple-map r1 r0 ,@table-syms)))
+    (if nestedp
+	`(loop
+	  (r1 = 0)
+	  (r4 = r0)
+	  (loop
+	   (map-multiple r1 r0 ,table-syms)
+	   (if (r0 > ,(mucs-max-number))
+	       ((if (r0 == ,(mucs-special-code 1))
+		    ((r0 = r4)
+		     (r1 += 1)
+		     (repeat))
+		  ((r0 += (r4 - ,(mucs-arithmetic-adjust)))
+		   (r1 += 1)
+		   (repeat)))))))
       `(loop
 	(r1 = 0)
-	(loop
-	 (r4 = r0)
-	 (map-multiple r1 r0 ,table-syms)
-	 (if (r0 > ,(mucs-max-code))
-	     ((r0 += (r4 - ,(mucs-arithmetic-adjust)))
-	      (r1 += 1)
-	      (repeat)))))
-    `(loop
-      (r1 = 0)
-      (r4 = r0)
-      (iterate-multiple-map r1 r0 ,@table-syms)
-      (if (r0 > ,(mucs-max-code))
-	  ((r0 += (r4 - ,(mucs-arithmetic-adjust))))))))
+	(r4 = r0)
+	(iterate-multiple-map r1 r0 ,@table-syms)
+	(if (r0 > ,(mucs-max-number))
+	    ((r0 += (r4 - ,(mucs-arithmetic-adjust)))))))))
 
 ;; API for compilation.
 
@@ -1651,7 +1720,11 @@ composite-transparent operation is disabled intentionally.")
 NAME must be a declared translation."
   (if (not (tae-declared-translation-p name))
       (error "Undeclared or invalid translation:%S" name))
-  (let* ((type (or applied-type
+  (let* ((tae-enable-register-translation
+	  (if mucs-current-package
+	      tae-enable-register-translation
+	    nil))
+	 (type (or applied-type
 		   mucs-current-type))
 	 (intdata (tae-compile-internal name decodep type))
 	 (result-myo (car intdata))
