@@ -44,6 +44,60 @@
   "Insert character which is converted from give UCS codepoint."
   t)
 
+;;; un-define post-read-conversion and pre-write-conversion.
+
+(defvar un-define-enable-buffer-conversion t
+  "* non-nil means the UTF-* coding-systems converts buffer contents
+by calling post-read-conversion and pre-write-conversion functions.")
+
+(defvar un-define-post-read-conversion-charsets-alist
+  '((thai-tis620 . thai-post-read-conversion)
+    ;; in-is13194-devanagari-post-read-conversion does not works correctly.
+    ;; I disabled the below line.
+    ;; (indian-is13194 . in-is13194-devanagari-post-read-conversion))
+    ))
+
+(defvar un-define-pre-write-conversion-charsets-alist
+  '(;; Disabled because read-multibyte-character
+    ;; decompose composite characters
+    ;; (thai-tis620 . thai-pre-write-conversion)
+    (indian-is13194 . in-is13194-devanagari-pre-write-conversion)
+    (indian-1-column . in-is13194-devanagari-pre-write-conversion)
+    (indian-2-column . in-is13194-devanagari-pre-write-conversion)))
+
+(defun un-define-post-read-conversion (len)
+  (if un-define-enable-buffer-conversion
+      (let* ((curpos (point))
+	     (charsets (find-charset-region curpos (+ curpos len)))
+	     slot func applied)
+	(while charsets
+	  (setq slot (assq (car charsets)
+			   un-define-post-read-conversion-charsets-alist)
+		func (cdr slot)
+		charsets (cdr charsets))
+	  (if (and slot
+		   (not (memq func applied)))
+	      (progn
+		(setq len (funcall func len)
+		      applied (cons func applied)))))))
+  len)
+
+(defun un-define-pre-write-conversion (from to)
+  (if un-define-enable-buffer-conversion
+      (let ((charsets (find-charset-region from to))
+	    slot func applied)
+	(while charsets
+	  (setq slot (assq (car charsets)
+			   un-define-pre-write-conversion-charsets-alist)
+		func (cdr slot)
+		charsets (cdr charsets))
+	  (if (and slot
+		   (not (memq func applied)))
+	      (progn
+		(funcall func from to)
+		(setq applied (cons func applied)))))))
+  nil)
+
 ;;; Required for compile, and also persistent section.
 (eval-and-compile
 
@@ -77,16 +131,37 @@
       korean-ksc5601
       latin-jisx0201
       katakana-jisx0201
-      mule-unicode-0100-24ff
-      thai-tis620) ;;; I'll give high priority in the future. This is a test.
-    "*Charset order used by unicode-basic-translation")
+      thai-tis620
+      ethiopic
+      indian-is13194
+      chinese-sisheng
+      vietnamese-viscii-lower
+      vietnamese-viscii-upper
+      mule-unicode-0100-24ff)
+    "*Charset order used by unicode-basic-translation.")
+
+  (defvar un-define-safe-charsets-for-coding-systems
+    (append
+     unicode-basic-translation-charset-order-list
+     '(indian-1-column
+       indian-2-column)))
 
 ;
 ; Default translation rule symbol per charset.
 ;
+
+; Make symbol of translation rule for Unicode.
+  (mapcar
+   (lambda (cs)
+     (put cs
+	  'translation-rule-for-unicode
+	  (intern (format "unicode-%s-translation-rule" cs))))
+   unicode-basic-translation-charset-order-list)
+
   (defun unicode-get-translation-rule-from-charset (charset)
-    (intern (format "unicode-%s-translation-rule"
-		    (symbol-name charset))))
+    (or (get charset 'translation-rule-for-unicode)
+	(error "Charset:%S has no translation rule for Unicode"
+	       charset)))
 
 ;
 ; font encoder charset spec alist
@@ -97,26 +172,24 @@
      (lambda (cs)
        (cons cs
 	     `(list
-	       (mucs-ccl-convert-font-encoding
-		,(charset-dimension cs))
+	       (mccl-font-convert-font-encoding)
 	       (tae-compile 
 		(quote
 		 ,(unicode-get-translation-rule-from-charset cs)))
-	       (mucs-ccl-flat-code-to-font-encoding 2))))
+	       (mccl-font-flat-code-to-font-encoding 2))))
      unicode-basic-translation-charset-order-list))
+
 ;;
 ;; Configuration functions for convenience.
 ;;
 
   (defun un-define-decode-template (tr read write)
-    (require 'unicode)
     (mucs-ccl-stream-form
      (mucs-ccl-read 'ucs-generic read)
      (tae-compile tr t)
      (mucs-ccl-write write)))
 
   (defun un-define-encode-template (tr read write)
-    (require 'unicode)
     (mucs-ccl-stream-form
      (mucs-ccl-read 'char-1 read)
      (tae-compile tr nil)
@@ -125,6 +198,8 @@
   (defun un-define-get-coding-system-alist
     (coding-category &optional mime-charset charsets)
     (append
+     '((pre-write-conversion . un-define-pre-write-conversion)
+       (post-read-conversion . un-define-post-read-conversion))
      (if mime-charset
 	 (list (cons 'mime-charset mime-charset)))
      (if (and coding-category
@@ -132,14 +207,16 @@
 	 (list (cons 'coding-category coding-category)))
      (list (cons 'safe-charsets
 		 (or charsets
-		     unicode-basic-translation-charset-order-list)))))
+		     un-define-safe-charsets-for-coding-systems)))))
   )
 
 (eval-when-compile
+
   ;; predefined configuration
   ;; Any settings in here affect only
   ;; byte-compiled file.
   (require 'un-trbase)
+  (require 'mccl-font)
   (require 'utf))
 
 (mucs-define-package
@@ -443,8 +520,8 @@
 
  (mucs-define-conversion
   unicode-font-encoder
-  (font-codepoint . font-codepoint)
-  (0 ((mucs-ccl-font-encoder
+  font
+  (0 ((mccl-font-encoder
        unicode-font-encoder-charset-spec-alist))))
 
 ;;
