@@ -29,9 +29,51 @@
 ;;;   When using this module, you should require this
 ;;;   in eval-when-compile form.
 
+(require 'tae)
 (require 'unicode)
 (require 'mule-uni)
 
+;
+; Replacement configuration.
+;
+
+(defvar mucs-unicode-default-decode-replacement ??)
+(defvar mucs-unicode-default-encode-replacement ?\xFFFD)
+
+;;
+;; UCS replacement or ignore translation rule.
+;;
+
+(defvar unicode-not-found-to-replace-or-invalid-assoc
+  `(assoc (char-1 . ucs-generic)
+	  ( ,@(mapcar
+	       (lambda (x)
+		 (cons 'invalid x))
+	       unicode-ignore-characters)
+	      (all . ,mucs-unicode-default-encode-replacement)
+	      (,mucs-unicode-default-decode-replacement . all)))
+  "Translate any values to replacement character or invalid code.
+If you want to deal with untranslated character, use this translation rule.")
+
+(defvar unicode-not-found-to-invalid-assoc
+  '(assoc (char-1 . ucs-generic)
+	  ((all . invalid)
+	   (invalid . all)))
+  "Translate any values to invalid code.
+If you want to deal with untranslated character, use this translation rule.")
+
+;;
+;; Dealing with line separator problem.
+;;
+
+(defvar lf-vs-cr-assoc
+  `(assoc (char-1 . ucs-generic)
+	  ((?\xa . ,unicode-cr))))
+
+(defvar lf-vs-unicode-line-separator-assoc
+  `(assoc (char-1 . ucs-generic)
+	  ((?\xa . ,unicode-line-separator))))
+
 ;
 ; Loader for parts of translation rule.
 ;
@@ -64,7 +106,12 @@
     (korean-ksc5601 . uksc5601)
     (ipa . uipa)
     (mule-unicode-0100-24ff . nil)
-    (thai-tis620 . utis620)))
+    (thai-tis620 . utis620)
+    (indian-is13194 . uiscii)
+    (ethiopic . uethiopic)
+    (chinese-sisheng . usisheng)
+    (vietnamese-viscii-lower . uviscii)
+    (vietnamese-viscii-upper . uviscii)))
 
 (defun require-unicode-charset-data (charset)
   (let ((package (cdr (assq charset unicode-charset-library-alist))))
@@ -118,7 +165,7 @@
  `(| ,lf-vs-cr-assoc
      unicode-basic-stream-translation-rule) t)
 
-;; translation definition ends here
+;; translation rule definition ends here
 
 (provide 'un-trbase)
 
