@@ -40,7 +40,8 @@
 				"./lisp/unicode.el"
 				"./lisp/utf.el"
 				"./lisp/un-data.el"
-				"./lisp/un-tools.el"))
+				"./lisp/un-tools.el"
+				"./lisp/unidata.el"))
        (compile-file-list-2nd (if command-line-args-left
 				  (prog1
 				      command-line-args-left
